/*
T-SQL-Abfragen fr Microsoft SQL-Server 2022
Kontakt per E-Mail: info@it-angermann.de
Letzte Aktualisierung:  23 Mrz 2024.
Dirk Angermann
https://it-angermann.de
*/

-- Gesucht ist die Gesamtliefermenge.
SELECT
	SUM(LMenge) AS 'Summe der Liefermengen'
FROM dbo.Lieferung;

-- Gesucht sind der grte, der kleinste und der durchschnittliche 
-- Statuswert aller Lieferanten. 
SELECT 
	MAX([Status]) AS 'Maximum'
	,MIN([Status]) AS 'Minimum'
	,AVG([Status]) AS 'Durchschnitt' 
FROM dbo.Lieferant;

-- ***********************************************
-- * Wissenswertes zu COUNT(*) vs. COUNT(Spalte) *
-- ***********************************************

-- Erstellen einer Tabelle
CREATE TABLE dbo.Test
(
	Spalte1 INT NULL
);

-- Hinzufgen von 3 Datenstzen
INSERT INTO dbo.Test VALUES (NULL);
INSERT INTO dbo.Test VALUES (1);
INSERT INTO dbo.Test VALUES (NULL);

-- Die Abfrage mit COUNT(*) zhlt die Anzahl der Datenstze. Auch werden 
-- NULL-Werte gezhlt.
SELECT
	COUNT(*) AS 'Anzahl'
FROM dbo.Test;

-- Die Abfrage mit COUNT(Spalte) ergibt nur die Anzahl der tatschlichen
-- Werte.
SELECT 
	COUNT(Spalte1) AS 'Anzahl'
FROM dbo.Test;

-- Anzahl aller bisherigen Lieferungen, die Lieferanten durchgefhrt
-- haben.
SELECT COUNT(*) AS 'Anzahl bisheriger Lieferungen'
FROM dbo.Lieferung;

-- oder
SELECT COUNT(ALL LNr) AS 'Anzahl bisheriger Lieferungen'
FROM dbo.Lieferung;

-- oder
SELECT COUNT(LNr) AS 'Anzahl bisheriger Lieferungen'
FROM dbo.Lieferung

-- Anzahl der Lieferanten, die bisher geliefert haben.
SELECT COUNT(DISTINCT LNr) AS 'Anzahl von Lieferanten'
FROM dbo.Lieferung

-- **********************************************************************
-- * GROUP BY-Klausel -- Erstellt eine Gruppenbildungsklausel, nach der *
-- * die Datenstze, die die Bedingung der WHERE-Klausel *
-- * (oder ohne WHERE-Klausel) erfllt haben, gruppiert werden. *
-- **********************************************************************
-- Anzuwenden bei Aggregatfunktionen (AVG, MIN, MAX, COUNT, ...)
-- Aggregatfunktionen beziehen sich immer auf Spalten z.B. COUNT(ANr), 
-- SUM(LMenge), AVG(Gewicht), ...
-- Ausnahme: COUNT(*) auf die Anzahl der Datenstze

-- Gesucht ist zu jeder Artikelnummer die Gesamtliefermenge.
SELECT
	ANr AS 'Artikelnummer'
	,SUM(LMenge) AS 'Summe der Liefermengen'
FROM dbo.Lieferung;

-- Lsung: GROUP BY-Klausel
SELECT
	ANr AS 'Artikelnummer'
	,SUM(LMenge) AS 'Summe der Liefermengen'
FROM dbo.Lieferung
GROUP BY ANr;

-- Gesucht ist die grte Liefermenge derjenigen Lieferanten, die nach
-- dem 01.08.2019 geliefert haben.
SELECT 
	LNr AS 'Nummer des Lieferanten'
	,MAX(LMenge) AS 'grte Liefermenge'
FROM dbo.Lieferung
WHERE LDatum > '20190801'
GROUP BY LNr;


-- Gesucht ist der Lagerort und die Gesamtlagermenge am jeweiligen
-- Lagerort der Artikel.
SELECT
	AStadt AS 'Lagerort'
	,SUM(AMenge) AS 'Summe'
FROM dbo.Artikel
GROUP BY AStadt;

-- Wie oft wurde jeder Artikel geliefert?
SELECT
	ANr AS 'Nummer'
	,COUNT(ANr) AS 'Wie oft?'
FROM dbo.Lieferung
GROUP BY ANr;

-- Wie viele unterschiedliche Artikel wurden von jedem Lieferanten
-- geliefert?
SELECT
	LNr AS 'Nummer'
	,COUNT(DISTINCT ANr) AS 'Wie viele unterschiedliche ..?'
FROM dbo.Lieferung
GROUP BY LNr;

-- Anmerkung: Das Schlsselwort DISTINCT ist erforderlich, da es um die 
-- Anzahl der unterschiedlichen Artikel geht.

-- Das Durchschnittsgewicht der roten und grnen Artikel am jeweiligen 
-- Lagerort.
SELECT
	AStadt AS 'Lagerort'
	,Farbe
	,AVG(Gewicht) AS 'Durchschnittsgewicht'
FROM dbo.Artikel
WHERE Farbe IN ('rot','grn')
GROUP BY AStadt, Farbe;

-- *************************************************************
-- * HAVING-Klausel -- legt Bedingungen fr die Gruppen fest *
-- *************************************************************
-- Aggregatfunktionen drfen nicht in einer WHERE-Klausel stehen
-- Sie definiert Bedingungen fr Gruppen
-- Die HAVING-Klausel bentigt immer die GROUP BY-Klausel
-- Die HAVING-Klausel ist die WHERE-Klausel der GROUP BY-Klausel.

-- Gesucht ist der grte Status der Lieferanten am jeweiligen Wohnort,
-- wenn diese nicht in Aachen wohnen und der durchschnittliche Statuswert 
-- am jeweiligen Wohnort nicht kleiner ist als 2.
-- Lsungsansatz:
SELECT 
	LStadt AS 'Firmensitz' 
	,MAX(Status) AS 'Statusangabe'
FROM dbo.Lieferant
WHERE LStadt <> 'Aachen' AND AVG([Status]) >= 2
GROUP BY LStadt;

-- Fehler: In der WHERE-Klausel darf kein Aggregat auftreten, es sei
-- denn, es befindet sich in einer Unterabfrage, 
-- die in einer HAVING-Klausel oder einer Auswahlliste enthalten 
-- ist, und die Spalte, die aggregiert wird, ist ein uerer
-- Verweis.
-- Lsung:
SELECT 
	LStadt AS 'Firmensitz' 
	,MAX(Status) AS 'Statusangabe'
FROM dbo.Lieferant
WHERE LStadt <> 'Aachen'
GROUP BY LStadt
HAVING AVG([Status]) >= 2;

-- Gesucht ist die Artikelnummer und die Gesamtliefermenge eines jeden
-- Artikels, wenn die Gesamtliefermenge mehr als 500 Stck betrug.
SELECT 
	ANr AS 'Artikelnummer'
	,SUM(LMenge) AS 'Summe der Liefermenge'
FROM dbo.Lieferung
GROUP BY ANr
HAVING SUM(LMenge) > 500;

-- Nennen Sie die Artikelnummern der Artikel, die weniger als dreimal
-- geliefert wurden.
SELECT
	ANr AS 'Artikelnummer'
	,COUNT(ANr) AS 'Wie oft?'
FROM dbo.Lieferung
GROUP BY ANr
HAVING COUNT(ANr) < 3;

-- Gesucht sind die Lieferantennummern und die Gesamtliefermenge der 
-- Lieferanten, die seit dem 13.07.2019 eine Gesamtmenge von Artikeln von 
-- mehr als 600 Stck geliefert haben.
SELECT
	LNr AS 'Lieferantennummer'
	,SUM(LMenge) AS 'Gesamtliefermenge'
FROM dbo.Lieferung
WHERE LDatum >= '20190713'
GROUP BY LNr
HAVING SUM(LMenge) > 600;

